/*
 * Copyright (c) 2021.  Jeffrey Vroom. All Rights Reserved.
 */

package sc.obj;

/**
 * This interface is an alternative to IComponent used for classes with name conflicts on one of the methods in that interface.
 * Having short, standard names for those methods is important but for the rare clash (e.g. RSyntaxTextArea which defines a protected init() 
 * method which is called in the constructor (which is too soon)) you can set CompilerSettings(useAltComponent=true).  The class will
 * implement this interface instead with the same semantics. 
 */
public interface IAltComponent extends IAltStoppable {
   public byte _getInitState();

   // Generated by the system: contains initialization for any variables which reference other objects, called
   // after the constructor.
   public void _preInit();

   /** You can optionally implement this method to receive a initialization hook */
   public void _init();

   /** You can optionally implement this method to receive a start hook */
   public void _start();
}
